/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck;

import static java.lang.String.format;

abstract class AbstractRunner<T> implements Runner<T> {

	void print(String msg) {
		getWriter().println(msg);
		getWriter().flush();
	}

	void printCurrentValue(int currentRuns, T value) {
		print(format("%s:[%s]", currentRuns, value));
	}

	void throwFailedException(T value, Throwable e, Characteristic<?> characteristic, int currentRuns) {
		String characteristicName = characteristic.name() == null ? "" : characteristic.name();
		String message = "Characteristic %s is not true for value(# %s):[%s] message: >%s<";
		String msg = format(message, characteristicName, currentRuns, value, e.getMessage());
		print(msg);
		throw new CharacteristicException(msg, e, characteristic, value);
	}
}