package net.java.quickcheck.junit.support;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import net.java.quickcheck.Generator;
import net.java.quickcheck.junit.ForAll;

public class GeneratorFactory {

	public static List<Generator<Object>> createGenerator(
			Object testClassInstance, ForAll annotation) {
		if (Discovery.isGeneratorMethodAnnotation(annotation)) {
			return createGeneratorFromMethod(testClassInstance, annotation);
		} 
		return createGeneratorFromClass(annotation.generatorClass());
	}

	/**
	 * @param testClassInstance
	 * @param annotation
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private static List<Generator<Object>> createGeneratorFromMethod(
			Object testClassInstance, ForAll annotation) {

		List<Method> generatorMethods = Discovery.getGeneratorMethods(
				testClassInstance.getClass(), annotation.generatorMethod());
		List<Generator<Object>> generators = new ArrayList<Generator<Object>>();
		for (Method generatorMethod : generatorMethods) {
			try {
				generators.add((Generator<Object>) generatorMethod.invoke(
						testClassInstance));
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
		return generators;
	}

	@SuppressWarnings("unchecked")
	private static List<Generator<Object>> createGeneratorFromClass(
			Class<?>[] generatorClasses) {
		List<Generator<Object>> gens = new ArrayList<Generator<Object>>();
		for (Class<?> generatorClass : generatorClasses) {
			gens.add((Generator<Object>) newInstance(generatorClass));
		}
		return gens;
	}

	// TODO move to own class
	public static Object newInstance(Class<?> clazz) {
		try {
			return clazz.newInstance();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
