/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck.junit3;

import junit.framework.TestCase;
import net.java.quickcheck.characteristic.AbstractCharacteristic;

/**
 * <p>
 * JUnit integration for Quickcheck.
 * </p>
 * <p>
 * For each execution of {@link Characteristic#specify(Object)} TestCase.setup()
 * and TestCase.tearDown() will be called.
 * </p>
 */
public abstract class AbstractQuickCheckTestCase extends TestCase {

	@Override
	public void runBare() throws Throwable {
		runTest();
	}

	public abstract class Characteristic<T> extends AbstractCharacteristic<T> {
		@Override
		public void setUp() throws Exception {
			AbstractQuickCheckTestCase.this.setUp();
		}

		@Override
		public void tearDown() throws Exception {
			AbstractQuickCheckTestCase.this.tearDown();
		}
	}

}
