/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck.junit4;

import net.java.quickcheck.junit.ForAll;

import org.junit.Test;
import org.junit.internal.runners.CompositeRunner;
import org.junit.internal.runners.InitializationError;
import org.junit.runner.Runner;

/**
 * <p>
 * Add the annotation @RunWith(QuickCheckRunner.class) to execute test with the
 * Quickcheck runner.
 * </p>
 * The runner runs tests annotated with the Junit 4 {@link Test} annotation and
 * the Quickcheck {@link ForAll} annotation.
 * 
 * @author thomas
 * 
 */
public class QuickCheckRunner extends CompositeRunner {
	public QuickCheckRunner(Class<?> testClass) throws InitializationError {
		super(testClass.getCanonicalName());
		addIfNotEmpty(new OnlyQuickCheckRunner(testClass));
		addIfNotEmpty(new MissingTestMethodIgnoringJUnit4ClassRunner(testClass));
	}

	/**
	 * @param runner
	 */
	private void addIfNotEmpty(Runner runner) {
		if (!runner.getDescription().getChildren().isEmpty()) {
			add(runner);
		}
	}
}